/*
 * Copyright 1993-2006 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:   
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and 
 * international Copyright laws.  
 *
 * This software and the information contained herein is PROPRIETARY and 
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and 
 * conditions of a Non-Disclosure Agreement.  Any reproduction or 
 * disclosure to any third party without the express written consent of 
 * NVIDIA is prohibited.     
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE 
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH 
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF 
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.   
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE 
 * OR PERFORMANCE OF THIS SOURCE CODE.  
 *
 * U.S. Government End Users.  This source code is a "commercial item" as 
 * that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of 
 * "commercial computer software" and "commercial computer software 
 * documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995) 
 * and is provided to the U.S. Government only as a commercial end item.  
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through 
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the 
 * source code with only those rights set forth herein.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

////////////////////////////////////////////////////////////////////////////////
// export C interface
extern "C" 
void computeGold( float* reference, float* idata, const unsigned int len);

////////////////////////////////////////////////////////////////////////////////
//! Compute reference data set
//! Each element is the sum of the elements before it in the array.
//! @param reference  reference data, computed but preallocated
//! @param idata      input data as provided to device
//! @param len        number of elements in reference / idata
////////////////////////////////////////////////////////////////////////////////

int floatCompare(const void* a, const void* b)
{
  union{ 
      float result; 
      int retval; 
  }value;
  value.result = (*(float*)a > *(float*)b); 
  return value.retval;
}

void
computeGold( float* reference, float* idata, const unsigned int len) 
{
  memcpy(reference, idata, len*sizeof(float));
  qsort(reference, len, sizeof(float), floatCompare); 
}



/*
     SHELL SORT
     Written by Sanchit Karve AKA born2c0de
     born2c0de AT hotmail DOT com
*/
 
/* 
#include <stdio.h>
 
#define ELEMENTS 26000
 */
void shellsort1(float A[],int max)
{
     int stop,swap,limit;
  float temp;
   int gap=(int)(max/2)-1;
   while(gap>0)
   {
        stop=0;
      limit=max-gap;
      while(stop==0)
      {
           swap=0;
         for(int k=0;k<limit;k++)
         {
              if(A[k]>A[k+gap])
            {
                 temp=A[k];
               A[k]=A[k+gap];
               A[k+gap]=temp;
               swap=k;
            }
         }
         limit=swap-gap;
         if(swap==0)
              stop=1;
      }
      gap=(int)(gap/2);
   }
}
/* 
int main()
{
  int i;
  float X[ELEMENTS];//={5,2,4,6,1,3};
  for(i=0;i<ELEMENTS;i++)
            X[i]=ELEMENTS-i;
  //for(i=0;i<ELEMENTS;i++)
  //          printf("%0.3f ",X[i]);
 
  shellsort(X,ELEMENTS);
  printf("\nSORTED ARRAY\n");
  //for(i=0;i<ELEMENTS;i++)
 //          printf("%0.1f ",X[i]);
  getchar();
 
}
 */
 
/*
http://www.unix.com/high-level-programming/8858-comb-shell-sort.html
//Source Code
//Below is the basic shell sort algorithm. 

void shellSort(int numbers[], int array_size)
{
int i, j, increment, temp;
increment = 3;
while (increment > 0)
{
for (i=0; i < array_size; i++)
{
j = i;
temp = numbers[i];
while ((j >= increment) && (numbers[j-increment] > temp))
{
numbers[j] = numbers[j - increment];
j = j - increment;
}
numbers[j] = temp;
}
if (increment/2 != 0)
increment = increment/2;
else if (increment == 1)
increment = 0;
else
increment = 1;
}
}
*/
/* 
#include <stdio.h> 
#include <stdlib.h> 
#include <time.h> 
#define MAX 10 

void shellsort(int[]); 
int main(void) { 
    int number[MAX] = {0}; 
    int i;  
    srand(time(NULL)); 
    printf("ƧǫeG"); 
    for(i = 0; i < MAX; i++) { 
        number[i] = rand() % 100; 
        printf("%d ", number[i]); 
    } 
    shellsort(number); 
    return 0; 
} 
*/
#define SWAP(x,y) {float t; t = x; x = y; y = t;} 
int floatCompare(const void* a, const void* b)
{
  union{ 
      float result; 
      int retval; 
  }value;
  value.result = (*(float*)a > *(float*)b); 
  return value.retval;
}
void shellsort2(float number[], int MAX) { 
    int i, j, k, gap, t; 
//    gap = MAX / 2; 
//	gap = MAX;
//    while(gap > 0) { 
//	printf("\ngap = %d\n\r", gap);   //gap = MAX / 2; gap = 1; gap --
//	printf("\ngap = %d\n\r", gap); 
	for (gap = MAX / 2; gap > 0; gap /= 2){
        for(k = 0; k < gap; k++) { 
            for(i = k+gap; i < MAX; i+=gap) { 
                for(j = i - gap; j >= k; j-=gap) { 
                    if(number[j] > number[j+gap]) { 
                        SWAP(number[j], number[j+gap]); 
                    } 
                    else 
                        break; 
                } 
            } 
        } 
//        printf("\ngap = %dG", gap); 
//		getchar();
//        for(i = 0; i < MAX; i++) 
//            printf("%d ", number[i]); 
//        printf("\n"); 
    //    gap /= 2; 
    } 
} 

 